# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.autocomplete.tests
import os
import z3c.form.form
import z3c.form.tests
import zope.app.testing.functional
import zope.component


class FunctionalLayer(zope.app.testing.functional.ZCMLLayer):
    def setUp(self):
        zope.app.testing.functional.ZCMLLayer.setUp(self)
        zope.component.provideAdapter(z3c.form.form.FormTemplateFactory(
            os.path.join(os.path.dirname(gocept.autocomplete.tests.__file__),
                         'layout.pt')))


ftesting_zcml = os.path.join(os.path.dirname(__file__), 'ftesting.zcml')
functional_layer = FunctionalLayer(
    ftesting_zcml, __name__, 'FunctionalLayer',
    allow_teardown=True)



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

from zope.traversing.api import traverse
import gocept.autocomplete.testing
import gocept.autocomplete.tests.color
import gocept.autocomplete.widget
import unittest
import z3c.form.interfaces
import z3c.form.testing
import zope.app.testing.functional
import zope.interface
import zope.publisher.browser
import zope.schema
import zope.traversing.adapters
import zope.traversing.interfaces


class WidgetTest(zope.app.testing.functional.FunctionalTestCase):
    layer = gocept.autocomplete.testing.functional_layer

    def setUp(self):
        super(WidgetTest, self).setUp()
        z3c.form.testing.setupFormDefaults()

    def test_sources_value_are_not_converted(self):
        request = z3c.form.testing.TestRequest()
        house = gocept.autocomplete.tests.color.House()
        house.color = u"red"
        form = gocept.autocomplete.tests.color.HouseForm(house, request)
        form.update()
        self.assertEqual(u"red", form.widgets['color'].value)

        request.form[form.widgets['color'].name] = u"foo"
        form.handleApply(form, None)
        self.assertEqual(u"foo", house.color)

    def test_traversal(self):
        request = z3c.form.testing.TestRequest()
        house = gocept.autocomplete.tests.color.House()
        zope.component.provideAdapter(
            zope.traversing.adapters.RootPhysicallyLocatable,
            (gocept.autocomplete.tests.color.House,),
            zope.traversing.interfaces.IPhysicallyLocatable)
        house.color = u"red"
        form = gocept.autocomplete.tests.color.HouseForm(house, request)
        zope.component.provideAdapter(
            lambda x,y: form,
            (gocept.autocomplete.tests.color.House, z3c.form.testing.TestRequest),
            zope.interface.Interface,
            name='form'
            )
        # we intentionally don't call form.update() ourselves,
        # since the traverser shouldn't assume that it has been called
        actual = traverse(house, '/@@form/++widget++color', request=request)
        self.assertEqual(form.widgets['color'], actual)



def test_suite():
    return unittest.makeSuite(WidgetTest)



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.autocomplete.interfaces
import os
import z3c.form.field
import z3c.form.form
import z3c.form.interfaces
import z3c.form.tests
import zope.app.appsetup.bootstrap
import zope.component
import zope.interface
import zope.publisher.interfaces.browser
import zope.schema
import transaction


class ColorSource(object):
    zope.interface.implements(gocept.autocomplete.interfaces.ISearchableSource)

    _data = [u"red", u"blue", u"ruby"]

    def __iter__(self):
        for item in self._data:
            yield item

    def __contains__(self, value):
        return True

    def search(self, prefix):
        return [item for item in self
                if item.lower().find(prefix.lower()) == 0]


class IHouse(zope.interface.Interface):
    color = zope.schema.Choice(title=u"Color", source=ColorSource())


class House(object):
    zope.interface.implements(IHouse)

    color = None


class HouseForm(z3c.form.form.EditForm):
    fields = z3c.form.field.Fields(IHouse)

    def __call__(self, *args, **kw):
        return super(HouseForm, self).__call__(*args, **kw)


class IColorSkin(z3c.form.interfaces.IFormLayer,
                 zope.publisher.interfaces.browser.IDefaultBrowserLayer):
    pass


def init_demo(event):
    db, connection, root, root_folder = (
        zope.app.appsetup.bootstrap.getInformationFromEvent(event))

    zope.component.provideAdapter(z3c.form.form.FormTemplateFactory(
        os.path.join(os.path.dirname(gocept.autocomplete.tests.__file__),
                     'layout.pt')))

    root_folder['demo'] = House()
    transaction.commit()
    connection.close()



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import zope.component
import zope.interface
import zope.security.proxy
import zope.traversing.interfaces


class WidgetTraversable(object):
    """Traverser from a z3c.form to its widgets.

    /context/@@form/++widget++fieldname is the widget belonging to the form
    field 'fieldname'.
    """

    zope.interface.implements(zope.traversing.interfaces.ITraversable)

    def __init__(self, context, request):
        # XXX security!!
        self.context = zope.security.proxy.removeSecurityProxy(context)
        self.request = request

    def traverse(self, name, remaining):
        form = self.context
        form.update()
        return form.widgets[name]




# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt
#namespace package boilerplate
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError, e:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import z3c.form.interfaces
import zope.schema.interfaces


class IAutocompleteWidget(z3c.form.interfaces.IWidget):
    """Autocomplete widget"""


class ISearchableSource(zope.schema.interfaces.IIterableSource):
    """A source suitable for autocompletion.
    Note that its __contains__() method should always return True.
    """

    def search(prefix):
        """Returns a list of values from this source that match the given
        prefix."""



import gocept.autocomplete.testing
import zope.testing.doctest


optionflags = (zope.testing.doctest.REPORT_NDIFF
               | zope.testing.doctest.NORMALIZE_WHITESPACE
               | zope.testing.doctest.ELLIPSIS)


def test_suite():
    suite = zope.testing.doctest.DocFileSuite('README.txt',
                                              package='gocept.autocomplete',
                                              optionflags=optionflags)
    suite.layer = gocept.autocomplete.testing.functional_layer
    return suite



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt
"""
Since we want to accept any value that the user entered, even if it is not
part of the source yet, we do not use terms/tokens as usual but rather use
the value itself for everything.
"""

import gocept.autocomplete.widget
import z3c.form.converter
import zope.browser.interfaces
import zope.component
import zope.interface
import zope.publisher.interfaces.browser
import zope.schema.interfaces


class IdentityTerm(object):
    zope.interface.implements(zope.schema.interfaces.ITitledTokenizedTerm)

    def __init__(self, value):
        self.value = value
        self.token = value
        self.title = value


class IdentityTerms(object):
    zope.interface.implements(zope.browser.interfaces.ITerms)
    zope.component.adapts(gocept.autocomplete.interfaces.ISearchableSource,
                          zope.publisher.interfaces.browser.IBrowserRequest)

    def __init__(self, source, request):
        pass

    def getTerm(self, value):
        return IdentityTerm(value)

    def getValue(self, token):
        return token


class SourceDataConverter(z3c.form.converter.BaseDataConverter):
    zope.component.adapts(zope.schema.interfaces.IChoice,
                          gocept.autocomplete.interfaces.IAutocompleteWidget)

    def toWidgetValue(self, value):
        return zope.component.getMultiAdapter(
            (self.field.source, self.widget.request),
            zope.browser.interfaces.ITerms).getTerm(value).token

    def toFieldValue(self, value):
        return zope.component.getMultiAdapter(
            (self.field.source, self.widget.request),
            zope.browser.interfaces.ITerms).getValue(value)




# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.autocomplete.interfaces
import string
import z3c.form.browser.text
import z3c.form.converter
import z3c.form.interfaces
import z3c.form.widget
import zc.resourcelibrary
import zope.interface
import zope.pagetemplate.interfaces
import zope.publisher.browser
import zope.security.proxy


class AutocompleteWidget(z3c.form.browser.text.TextWidget):
    zope.interface.implements(
        gocept.autocomplete.interfaces.IAutocompleteWidget)

    _javascript = """
YAHOO.namespace('gocept.autocomplete');

YAHOO.gocept.autocomplete.init_${id_} = new function() {
    this.datasource = new YAHOO.widget.DS_XHR("${url}", ["\\n"]);
    this.datasource.responseType = YAHOO.widget.DS_XHR.TYPE_FLAT;
    this.datasource.scriptQueryParam = "q";
    this.autocomp = new YAHOO.widget.AutoComplete(
        "${id}", "${id}-container", this.datasource);
    this.autocomp.prehighlightClassName = "yui-ac-prehighlight";
    this.autocomp.typeAhead = true;
    this.autocomp.useShadow = true;
    this.autocomp.doBeforeExpandContainer = function(textbox, container, query, results) {
        var pos = YAHOO.util.Dom.getXY(textbox);
        pos[1] += YAHOO.util.Dom.get(textbox).offsetHeight + 2;
        YAHOO.util.Dom.setXY(container, pos);
        return true;
    };
}

YAHOO.util.Event.onDOMReady(YAHOO.gocept.autocomplete.init_${id_});
"""

    def __init__(self, *args, **kw):
        super(AutocompleteWidget, self).__init__(*args, **kw)
        self.addClass(u'autocomplete')

    def render(self):
        zc.resourcelibrary.need("yui-autocomplete")
        return super(AutocompleteWidget, self).render()

    def input_field(self):
        class Dummy(object):
            pass
        parent = Dummy()
        zope.interface.alsoProvides(parent, z3c.form.interfaces.ITextWidget)
        super_template = zope.component.getMultiAdapter(
            (self.context, self.request, self.form, self.field,
             parent),
            zope.pagetemplate.interfaces.IPageTemplate, name=self.mode)
        return super_template(self)

    def javascript(self):
        context_url = str(zope.component.getMultiAdapter(
            (self.form.context, self.request), name='absolute_url'))

        search_url = "%s/@@%s/++widget++%s/@@autocomplete-search" % (
            context_url, self.form.__name__, self.name.split('.')[-1])

        return string.Template(self._javascript).substitute(dict(
            id=self.id, id_=self.id.replace('-', '_'), url=search_url))


@zope.component.adapter(zope.schema.interfaces.IChoice,
                        gocept.autocomplete.interfaces.ISearchableSource,
                        z3c.form.interfaces.IFormLayer)
@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def AutocompleteFieldWidget(field, source, request):
    return z3c.form.widget.FieldWidget(field, AutocompleteWidget(request))


class SearchView(zope.publisher.browser.BrowserView):
    def __call__(self):
        # XXX security!!
        context = zope.security.proxy.removeSecurityProxy(self.context)
        query = self.request.get("q")
        if query:
            return u"\n".join(context.field.source.search(query))
        else:
            return u""



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import subprocess
import sys
import webbrowser
import xml.sax.saxutils
import zc.selenium.pytest


if sys.platform == 'darwin':
    # Register a Firefox browser for Mac OS X.
    class MacOSXFirefox(webbrowser.BaseBrowser):
        def open(self, url, new=0, autoraise=1):
            proc = subprocess.Popen(
                ['/usr/bin/open', '-a', 'Firefox', url])
            proc.communicate()
    webbrowser.register('Firefox', MacOSXFirefox, None, -1)


class AutocompleteTest(zc.selenium.pytest.Test):
    def test_autocomplete(self):
        s = self.selenium

        # XXX: logging in this way on /demo directly (which does not *require*
        # login) does not work
        s.open('http://mgr:mgrpw@%s/manage' % self.selenium.server)

        s.open('/demo')
        s.comment('Typing a key should cause the word to be completed')
        # XXX: this *looks* like we're entering 'rr' (when one observes the
        # browser), but it does the right thing -- and all other combination
        # of calls I tried didn't work at all. :-(
        s.type('id=form-widgets-color', 'r')
        s.typeKeys('id=form-widgets-color', 'r')
        s.waitForValue('id=form-widgets-color', 'red')
        s.verifyText('id=form-widgets-color-container', '*red*')



# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import os.path
from setuptools import setup, find_packages


setup(
    name='gocept.autocomplete',
    version = '0.3',
    author='gocept',
    author_email='mail@gocept.com',
    description='AJAX autocomplete widget for z3c.form',
    long_description = (
        open(os.path.join('src', 'gocept', 'autocomplete', 'README.txt')).read() +
        '\n\n' +
        open('CHANGES.txt').read()),
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data=True,
    zip_safe=False,
    license='ZPL 2.1',
    namespace_packages=['gocept'],
    install_requires=[
        'setuptools',
        'ore.yui',
        'z3c.form',
        'zc.resourcelibrary',
        'zope.browser',
        'zope.component',
        'zope.interface',
        'zope.pagetemplate',
        'zope.publisher',
        'zope.schema',
        'zope.traversing',
        ],
    extras_require=dict(
        test=[
            'lxml',
            'zc.selenium',
            'zope.app.appsetup',
            'zope.app.testing',
            'zope.app.zcmlfiles',
            'zope.securitypolicy',
            'zope.testing',
            'zope.testbrowser>=3.4.3',
            ]),
)
